/* Name: main.c
 * Project: hid-mouse, a very simple HID example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-07
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.

We use VID/PID 0x046D/0xC00E which is taken from a Logitech mouse. Don't
publish any hardware using these IDs! This is for demonstration only!
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

//PROGMEM const char usbHidReportDescriptor[52] = { /* USB report descriptor, size must match usbconfig.h */
//    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
//    0x09, 0x02,                    // USAGE (Mouse)
//    0xa1, 0x01,                    // COLLECTION (Application)
//    0x09, 0x01,                    //   USAGE (Pointer)
//    0xA1, 0x00,                    //   COLLECTION (Physical)
//    0x05, 0x09,                    //     USAGE_PAGE (Button)
//    0x19, 0x01,                    //     USAGE_MINIMUM
//    0x29, 0x03,                    //     USAGE_MAXIMUM
//    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
//    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
//    0x95, 0x03,                    //     REPORT_COUNT (3)
//    0x75, 0x01,                    //     REPORT_SIZE (1)
//    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
//    0x95, 0x01,                    //     REPORT_COUNT (1)
//    0x75, 0x05,                    //     REPORT_SIZE (5)
//    0x81, 0x03,                    //     INPUT (Const,Var,Abs)
//    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
//    0x09, 0x30,                    //     USAGE (X)
//    0x09, 0x31,                    //     USAGE (Y)
//    0x09, 0x38,                    //     USAGE (Wheel)
//    0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
//    0x25, 0x7F,                    //     LOGICAL_MAXIMUM (127)
//    0x75, 0x08,                    //     REPORT_SIZE (8)
//    0x95, 0x03,                    //     REPORT_COUNT (3)
//    0x81, 0x06,                    //     INPUT (Data,Var,Rel)
//    0xC0,                          //   END_COLLECTION
//    0xC0,                          // END COLLECTION
//};


PROGMEM const char usbHidReportDescriptor[60] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x02,                    // USAGE (Mouse)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM (Button 1)
    0x29, 0x03,                    //     USAGE_MAXIMUM (Button 3)
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x75, 0x05,                    //     REPORT_SIZE (5)
    0x81, 0x03,                    //     INPUT (Cnst,Var,Abs)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x35, 0x00,                    //     PHYSICAL_MINIMUM (0)
    0x46, 0x9d, 0x0b,              //     PHYSICAL_MAXIMUM (2973)
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x26, 0x9d, 0x0b,              //     LOGICAL_MAXIMUM (2973)
    0x65, 0x11,                    //     UNIT (SI Lin:Distance)
    0x55, 0x0e,                    //     UNIT_EXPONENT (-2)
    0x75, 0x10,                    //     REPORT_SIZE (16)
    0x95, 0x02,                    //     REPORT_COUNT (2)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0xc0,                          //   END_COLLECTION
    0xc0                           // END_COLLECTION
};


/* This is the same report descriptor as seen in a Logitech mouse. The data
 * described by this descriptor consists of 4 bytes:
 *      .  .  .  .  . B2 B1 B0 .... one byte with mouse button states
 *     X7 X6 X5 X4 X3 X2 X1 X0 .... 8 bit signed relative coordinate x
 *     Y7 Y6 Y5 Y4 Y3 Y2 Y1 Y0 .... 8 bit signed relative coordinate y
 *     W7 W6 W5 W4 W3 W2 W1 W0 .... 8 bit signed relative coordinate wheel
 */
/*
typedef struct{
    uchar   buttonMask;
    char    dx;
    char    dy;
    char    dWheel;
}report_t;
*/
typedef struct{
 	uchar		buttonMask;
	uint16_t	dx ;
	uint16_t	dy ;
    //    char    dWheel;
}report_t;

static report_t reportBuffer;
static int      sinus = 7 << 6, cosinus = 0;
static uchar    idleRate;   /* repeat rate for keyboards, never used for mice */
static uint32_t currentRegValue = 0;
static int16_t currentXContent = 0;
static int16_t currentYContent = 0;
static uint8_t delay  = 0;
static uint32_t startupdelay=100;
//#define arraylen = 29;
static uint8_t clickdown = 0;

PROGMEM uint8_t xhightable[236]={0, 0, 4, 4, 4, 4, 4, 4, 4, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 4, 4, 4, 4, 4, 4, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3,3,3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 3, 3, 3, 3, 3, 3, 4, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4};
PROGMEM uint8_t xlowtable[236]={42, 42, 135, 153, 169, 176, 175, 167, 153, 127, 47, 25, 11, 2, 3, 10, 24, 41, 41, 52, 52, 125, 143, 154, 158, 158, 154, 144, 125, 49, 34, 22, 17, 16, 24, 36, 50, 50, 123, 123, 72, 36, 36, 69, 124, 130, 124, 77, 53, 54, 78, 123, 131, 124, 124, 182, 182, 180, 171, 162, 162, 171, 181, 182, 213, 231, 236, 240, 249, 2, 255, 248, 234, 210, 181, 181, 49, 49, 114, 127, 132, 133, 127, 116, 45, 35, 31, 31, 36, 51, 51, 52, 52, 112, 111, 52, 52, 52, 180, 180, 242, 0, 4, 4, 0, 245, 174, 163, 158, 159, 165, 177, 177, 178, 178, 239, 239, 179, 179, 179, 62, 62, 49, 43, 50, 62, 63, 72, 81, 82, 106, 113, 105, 82, 83, 89, 124, 129, 122, 87, 75, 65, 62, 62, 181, 181, 166, 161, 165, 181, 181, 185, 198, 212, 235, 249, 1, 251, 239, 233, 207, 200, 200, 239, 246, 240, 201, 199, 190, 181, 181, 181, 52, 52, 109, 109, 96, 67, 52, 52,52,52, 52, 123, 129, 126, 99, 61, 34, 33, 59, 121, 129, 122, 66, 52, 52, 52, 250, 250, 195, 161, 163, 197, 249, 1, 248, 205, 180, 180, 204, 250, 0, 250, 250, 52, 52, 51, 41, 32, 32, 40, 51, 51, 83, 100, 106, 110, 119, 129, 125, 120, 107, 81, 51, 51};
PROGMEM uint8_t yhightable[236]={0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
PROGMEM uint8_t ylowtable[236]={1, 1, 2, 12, 29, 51, 165, 185, 200, 212, 211, 203, 187, 170, 41, 23, 10, 3, 3, 17, 17, 17, 26, 38, 52, 160, 174, 187, 195, 195, 190, 175, 161, 52, 33, 21, 17, 17, 76, 76, 76, 106, 151, 185, 184, 175, 167, 165, 147, 114, 95, 93, 86, 77, 77, 91, 91, 40, 35, 42, 178, 185, 175, 115, 94, 95, 101, 177, 185, 176, 94, 83, 75, 76, 91, 91, 26, 26, 26, 33, 48, 166, 179, 184, 185, 179, 167, 41, 32, 26, 26, 46, 46, 47, 165, 166, 47, 47, 27, 27, 26, 34, 47, 166, 179, 185, 184, 179, 167, 40, 31, 26, 26, 46, 46, 45, 164, 165, 46, 46, 74, 74, 76, 83, 95, 94, 178, 186, 177, 94, 94, 83, 75, 75, 62, 55, 54, 43, 35, 36, 42, 52, 74, 74, 75, 75, 76, 84, 93, 95, 160, 173, 183, 185, 185, 175, 158, 150, 153, 164, 166, 159, 95, 94, 84, 75, 75, 47, 41, 47, 74, 74, 124, 124, 124, 104, 95, 95, 105, 124,124,145, 145, 145, 136, 97, 77, 75, 99, 160, 185, 184, 176, 167, 166, 153, 145, 145, 76, 76, 77, 110, 154, 185, 185, 176, 166, 167, 146, 115, 95, 94, 84, 76, 76, 92, 92, 42, 35, 41, 177, 185, 178, 115, 94, 95, 103, 180, 185, 174, 96, 84, 76, 76, 92, 92};

//PROGMEM uint8_t xhightable[218]={3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,4,3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,3};
//PROGMEM uint8_t xlowtable[218]={7, 7, 15, 23, 32, 38, 38, 46, 46, 46, 46, 46, 46, 50, 56, 57, 57, 57, 66, 66, 69, 74, 78, 79, 79, 78, 78, 74, 69, 65, 65, 69, 74, 78, 78, 87, 87, 87, 91, 91, 83, 83, 89, 89, 107, 107, 107, 107, 117, 117, 116, 117, 121, 121, 136, 136, 136, 136, 145, 145, 144, 149, 149, 166, 166, 170, 176, 178, 178, 178, 178, 178, 174, 169, 164, 166, 173, 178, 178, 197, 197, 194, 187, 185, 187, 194, 198, 198, 205, 205, 205, 209, 209, 215, 215, 215, 218, 222, 227, 227, 227, 227, 227, 227, 235, 235, 239, 245, 248, 249, 249, 247, 247, 242, 236, 235, 237, 243, 248, 248, 0, 0, 0, 0, 10, 10, 9, 9, 17, 17, 22, 22, 30, 30, 19, 15, 15, 43, 43, 50, 55, 60, 67, 67, 81, 81, 85, 87, 82, 77, 72, 72, 81, 81, 93, 93, 93, 93, 94, 94, 98, 101, 101, 106, 106, 106, 106, 107, 107, 117, 117, 110, 110, 118, 118, 135, 135, 131, 126, 123, 128, 135, 136, 130, 126, 123, 123, 144, 144, 147, 153, 156, 154, 150, 149, 149, 149, 149, 149, 149,149,107, 107, 107, 107, 137, 137, 137, 137, 142, 142, 149, 149, 202, 202, 209, 209,209};
//PROGMEM uint8_t yhightable[218]={0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0};	
//PROGMEM uint8_t ylowtable[218]={230, 230, 0, 230, 255, 229, 229, 229, 229, 0, 0, 240, 240, 237, 238, 242, 254, 254, 242, 242, 238, 238, 240, 254, 254, 242, 242, 245, 245, 249, 252, 254, 253, 250, 250, 232, 232, 253, 255, 255, 238, 238, 238, 238, 238, 238, 255, 255, 255, 255, 235, 230, 230, 230, 237, 237, 0, 0, 232, 232, 253, 254, 254, 241, 241, 237, 237, 241, 255, 255, 242, 242, 245, 246, 250, 255, 253, 251, 251, 243, 243, 238, 239, 245, 253, 254, 250, 250, 231, 231, 252, 255, 255, 238, 238, 252, 254, 254, 250, 250, 238, 238, 255, 255, 241, 241, 238, 238, 240, 255, 255, 243, 243, 245, 246, 250, 254, 254, 249, 249, 231, 231, 0, 0, 231, 231, 0, 0, 237, 237, 253, 253, 238, 238, 5, 5, 5, 237, 237, 0, 236, 255, 237, 237, 237, 237, 241, 249, 0, 255, 250, 243, 238, 238, 237, 237, 255, 255, 239, 239, 238, 239, 239, 231, 231, 255, 255, 246, 246, 237, 237, 245, 245, 0, 0, 240, 240, 237, 237, 242, 245, 247, 251, 255, 254, 249, 249, 235, 235, 231, 231, 235, 242, 246, 250, 250, 255, 255, 255, 255,255,230, 230, 231, 231, 231, 231, 231, 231, 238, 238, 239, 239, 238, 238, 238, 238,238};
		
/* The following function advances sin/cos by a fixed angle
 * and stores the difference to the previous coordinates in the report
 * descriptor.
 * The algorithm is the simulation of a second order differential equation.
 */
static void advanceCircleByFixedAngle(void)
{
char    d;

#define DIVIDE_BY_64(val)  (val + (val > 0 ? 32 : -32)) >> 6    /* rounding divide */
    reportBuffer.dx = d = DIVIDE_BY_64(cosinus);
    sinus += d;
    reportBuffer.dy = d = DIVIDE_BY_64(sinus);
    cosinus -= d;
}

static void nextPoint(void)
{

    if (currentRegValue>0)
    {
	    if (pgm_read_byte(&xhightable[currentRegValue])==pgm_read_byte(&xhightable[currentRegValue-1])&&pgm_read_byte(&xlowtable[currentRegValue])==pgm_read_byte(&xlowtable[currentRegValue-1])&&pgm_read_byte(&yhightable[currentRegValue])==pgm_read_byte(&yhightable[currentRegValue-1])&&pgm_read_byte(&ylowtable[currentRegValue])==pgm_read_byte(&ylowtable[currentRegValue-1]))
	    {
		    if (clickdown)
		    {
			    clickdown = 0;
			    reportBuffer.buttonMask = 0;
		    }
		    else
		    {
			    reportBuffer.buttonMask = 1;
			    clickdown = 1;
		    }
	    }
    }
    if (currentRegValue<236)
    {
        
        reportBuffer.dx = 728 + (pgm_read_byte(&xhightable[currentRegValue])*256)+ pgm_read_byte(&xlowtable[currentRegValue]);
        reportBuffer.dy = 1496 +(pgm_read_byte(&yhightable[currentRegValue])*256) + pgm_read_byte(&ylowtable[currentRegValue]);
            currentRegValue++;
    }
    else
        currentRegValue=0;
}
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    /* The following requests are never used. But since they are required by
     * the specification, we implement them in this example.
     */
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
  //      DBG1(0x50, &rq->bRequest, 1);   /* debug output: print our request */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            usbMsgPtr = (void *)&reportBuffer;
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int __attribute__((noreturn)) main(void)
{
uchar   i;

    wdt_enable(WDTO_8S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    odDebugInit();
    DBG1(0x00, 0, 0);       /* debug output: main starts */
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    reportBuffer.dy = 1400;
    reportBuffer.dx = 1400;
    reportBuffer.buttonMask=0;
    for(;;){                /* main event loop */
        DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
        wdt_reset();
        usbPoll();
        
        if(usbInterruptIsReady()){
            /* called after every poll of the interrupt endpoint */
            //advanceCircleByFixedAngle();
            delay++;
	    
		if (startupdelay)
			startupdelay--;
		else
		{
			if (delay%1==0)
				nextPoint();
		}
            DBG1(0x03, 0, 0);   /* debug output: interrupt report prepared */
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }
}

/* ------------------------------------------------------------------------- */
